@@ -1,5 +1,5 @@
 #[cfg(target_os = "macos")]
-use crate::platform::macos::{CommandSource, IOKitRegistry};
+use crate::platform::macos::{CommandSource, IOKitRegistry, get_pid_responsible};
 #[cfg(target_os = "windows")]
 use crate::platform::windows::{EtwTrace, Pdh, ProcessorInfo, ThermalZoneInformation, VmCounter};
 use crate::{Error, Pid};
@@ -52,7 +52,9 @@ impl System {
             use_sysinfo_system = true;
         }
         if use_sysinfo_system {
-            system.sysinfo_system = Some(sysinfo::System::new_with_specifics(system.refresh_kind));
+            let mut sysinfo_system = sysinfo::System::new_with_specifics(system.refresh_kind);
+            sysinfo_system.refresh_specifics(system.refresh_kind);
+            system.sysinfo_system = Some(sysinfo_system);
         }
 
         if features.contains(Features::GPU) {
@@ -197,6 +199,25 @@ impl System {
         Some(self.sysinfo_system.as_ref()?.process(pid)?.cmd())
     }
 
+    pub fn process_responsible(&self, pid: Pid) -> Option<Pid> {
+        #[cfg(target_os = "macos")]
+        {
+            get_pid_responsible().map(|f| {
+                let pid_responsible = f(pid);
+                if pid_responsible < 0 {
+                    None
+                } else {
+                    Some(pid_responsible)
+                }
+            }).flatten()
+        }
+
+        #[cfg(target_os = "windows")]
+        {
+            None
+        }
+    }
+
     pub fn cpus_frequency(&self) -> Result<Vec<f32>, Error> {
         #[cfg(target_os = "macos")]
         {
